@@ -1,6 +1,22 @@
 // DasherInterfaceBase.cpp
 //
-// Copyright (c) 2002 Iain Murray
+// Copyright (c) 2007 The Dasher Team
+//
+// This file is part of Dasher.
+//
+// Dasher is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+//
+// Dasher is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Dasher; if not, write to the Free Software 
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "../Common/Common.h"
 
@@ -56,7 +72,7 @@ using namespace std;
 
 // Track memory leaks on Windows to the line that new'd the memory
 #ifdef _WIN32
-#ifdef _DEBUG
+#ifdef _DEBUG_MEMLEAKS
 #define DEBUG_NEW new( _NORMAL_BLOCK, THIS_FILE, __LINE__ )
 #define new DEBUG_NEW
 #undef THIS_FILE
@@ -64,26 +80,37 @@ static char THIS_FILE[] = __FILE__;
 #endif
 #endif
 
-CDasherInterfaceBase::CDasherInterfaceBase()
-                  :m_Alphabet(0), m_pDasherModel(0), m_DasherScreen(0),
-		   m_pDasherView(0), m_pInput(0), m_AlphIO(0), m_ColourIO(0), m_pUserLog(NULL),
-		   m_pInputFilter(NULL) {
+CDasherInterfaceBase::CDasherInterfaceBase() {
 
-  m_iCurrentState = ST_START;
-  m_iLockCount = 0;
+  // Ensure that pointers to 'owned' objects are set to NULL.
+  m_Alphabet = NULL;
+  m_pDasherModel = NULL;
+  m_DasherScreen = NULL;
+  m_pDasherView = NULL;
+  m_pInput = NULL;
+  m_AlphIO = NULL;
+  m_ColourIO = NULL;
+  m_pUserLog = NULL;
+  m_pInputFilter = NULL;
+  m_pNCManager = NULL;
 
-  m_pNCManager = 0;
+  // Various state variables
+  m_bRedrawScheduled = false;
   
+  m_iCurrentState = ST_START;
+  
+  m_iLockCount = 0;
   m_bGlobalLock = false;
 
-  m_bRedrawScheduled = false;
-
-  m_pEventHandler = new CEventHandler(this);
- 
+  // TODO: Are these actually needed?
   strCurrentContext = ". ";
-
   strTrainfileBuffer = "";
 
+  // Create an event handler.
+  m_pEventHandler = new CEventHandler(this);
+
+  m_bLastChanged = true;
+
   // Global logging object we can use from anywhere
   g_pLogger = new CFileLogger("dasher.log",
                               g_iLogLevel,
@@ -130,7 +157,7 @@ void CDasherInterfaceBase::Realize() {
   SetupActionButtons();
 
   // FIXME - need to rationalise this sort of thing.
-  InvalidateContext(true);
+  // InvalidateContext(true);
   ScheduleRedraw();
     
   // All the setup is done by now, so let the user log object know
@@ -138,11 +165,11 @@ void CDasherInterfaceBase::Realize() {
   if (m_pUserLog != NULL) 
     m_pUserLog->InitIsDone();
 
+  // TODO: Make things work when model is created latet
   ChangeState(TR_MODEL_INIT);
 }
 
 CDasherInterfaceBase::~CDasherInterfaceBase() {
-
   DASHER_ASSERT(m_iCurrentState == ST_SHUTDOWN);
 
   delete m_pDasherModel;        // The order of some of these deletions matters
@@ -240,7 +267,7 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
 	 SetStringParameter(SP_COLOUR_ID, GetStringParameter(SP_DEFAULT_COLOUR_ID));
       break;
     case LP_LANGUAGE_MODEL_ID:
-      CreateDasherModel();
+      CreateNCManager();
       break;
     case LP_LINE_WIDTH:
       ScheduleRedraw();
@@ -255,6 +282,9 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
       CreateInputFilter();
       ScheduleRedraw();
       break;
+    case BP_DASHER_PAUSED:
+      ScheduleRedraw();
+      break;
     default:
       break;
     }
@@ -315,17 +345,36 @@ void CDasherInterfaceBase::WriteTrainFilePartial() {
   strTrainfileBuffer = strTrainfileBuffer.substr(100);
 }
 
-void CDasherInterfaceBase::CreateDasherModel() 
-{
+void CDasherInterfaceBase::CreateModel(int iOffset) {
+  // Creating a model without a node creation manager is a bad plan
+  if(!m_pNCManager)
+    return;
+
+  if(m_pDasherModel) {
+    delete m_pDasherModel;
+    m_pDasherModel = 0;
+  }
+  
+  if(m_deGameModeStrings.size() == 0) {
+    m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, m_pDasherView, iOffset);
+  }
+  else {
+    m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, m_pDasherView, iOffset, true, m_deGameModeStrings[0]);
+  }
+}
+
+void CDasherInterfaceBase::CreateNCManager() {
+  // TODO: Try and make this work without necessarilty rebuilding the model
+
   if(!m_AlphIO)
     return;
 
-  // TODO: Move training into model?
-  // TODO: Do we really need to check for a valid language model?
   int lmID = GetLongParameter(LP_LANGUAGE_MODEL_ID);
-  if( lmID != -1 ) {
 
-    // Train the new language model
+  if( lmID == -1 ) 
+    return;
+
+ // Train the new language model
     CLockEvent *pEvent;
     
     pEvent = new CLockEvent("Training Dasher", true, 0);
@@ -334,6 +383,13 @@ void CDasherInterfaceBase::CreateDasherModel()
 
     AddLock(0);
 
+    int iOffset;
+
+    if(m_pDasherModel)
+      iOffset = m_pDasherModel->GetOffset();
+    else
+      iOffset = 0; // TODO: Is this right?
+
     // Delete the old model and create a new one
     if(m_pDasherModel) {
       delete m_pDasherModel;
@@ -346,13 +402,10 @@ void CDasherInterfaceBase::CreateDasherModel()
     }
       
     if(m_deGameModeStrings.size() == 0) {
-      m_pNCManager = new CNodeCreationManager(m_pEventHandler, m_pSettingsStore, false, "", m_AlphIO);
-      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this);
+      m_pNCManager = new CNodeCreationManager(this, m_pEventHandler, m_pSettingsStore, false, "", m_AlphIO);
     }
     else {
-      m_pNCManager = new CNodeCreationManager(m_pEventHandler, m_pSettingsStore, true, m_deGameModeStrings[0], m_AlphIO);
-      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, true, m_deGameModeStrings[0]);
-      //      m_deGameModeStrings.pop_front();
+      m_pNCManager = new CNodeCreationManager(this, m_pEventHandler, m_pSettingsStore, true, m_deGameModeStrings[0], m_AlphIO);
     }
 
     m_Alphabet = m_pNCManager->GetAlphabet();
@@ -376,34 +429,35 @@ void CDasherInterfaceBase::CreateDasherModel()
     pEvent = new CLockEvent("Training Dasher", false, 0);
     m_pEventHandler->InsertEvent(pEvent);
     delete pEvent;
-  }
 
-  Start();
-  Redraw(true);
+    ReleaseLock(0);
 
-  ReleaseLock(0);
+    // TODO: Eventually we'll not have to pass the NC manager to the model...
+    CreateModel(iOffset);
 }
 
-void CDasherInterfaceBase::Start() {
-  // TODO: Clarify the relationship between Start() and
-  // InvalidateContext() - I believe that they essentially do the same
-  // thing
-  PauseAt(0, 0);
-  if(m_pDasherModel != 0) {
-    m_pDasherModel->Start();
-  }
-  if(m_pDasherView != 0) {
-//     m_pDasherView->ResetSum();
-//     m_pDasherView->ResetSumCounter();
-//     m_pDasherView->ResetYAutoOffset();
-  }
+// void CDasherInterfaceBase::Start() {
+//   // TODO: Clarify the relationship between Start() and
+//   // InvalidateContext() - I believe that they essentially do the same
+//   // thing
+//   PauseAt(0, 0);
+//   if(m_pDasherModel != 0) {
+//     m_pDasherModel->Start();
+//   }
+//   if(m_pDasherView != 0) {
+// //     m_pDasherView->ResetSum();
+// //     m_pDasherView->ResetSumCounter();
+// //     m_pDasherView->ResetYAutoOffset();
+//   }
 
-  int iMinWidth;
+// TODO: Reimplement this sometime
+
+//   int iMinWidth;
   
-  if(m_pInputFilter && m_pInputFilter->GetMinWidth(iMinWidth)) {
-    m_pDasherModel->LimitRoot(iMinWidth);
-  }
-}
+//   if(m_pInputFilter && m_pInputFilter->GetMinWidth(iMinWidth)) {
+//     m_pDasherModel->LimitRoot(iMinWidth);
+//   }
+// }
 
 void CDasherInterfaceBase::PauseAt(int MouseX, int MouseY) {
   SetBoolParameter(BP_DASHER_PAUSED, true);
@@ -474,7 +528,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime, bool bForceRedraw) {
 	if(m_pInputFilter) {
 	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel, &vAdded, &iNumDeleted);
 	}
-	
+
 	if (iNumDeleted > 0)
 	  m_pUserLog->DeleteSymbols(iNumDeleted);
 	if (vAdded.size() > 0)
@@ -491,7 +545,15 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime, bool bForceRedraw) {
     }
   }
 
-  Redraw(bChanged || m_bRedrawScheduled || bForceRedraw);
+  // TODO: This is a bit hacky - we really need to sort out the redraw logic
+  if((!bChanged && m_bLastChanged) || m_bRedrawScheduled || bForceRedraw) {
+    m_pDasherView->Screen()->SetCaptureBackground(true);
+    m_pDasherView->Screen()->SetLoadBackground(true);
+  }
+
+  Redraw((bChanged || m_bLastChanged) || m_bRedrawScheduled || bForceRedraw);
+
+  m_bLastChanged = bChanged;
   m_bRedrawScheduled = false;
 
   // This just passes the time through to the framerate tracker, so we
@@ -512,6 +574,7 @@ void CDasherInterfaceBase::CheckRedraw() {
 /// otherwise we're wasting effort.
 
 void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
+
   if(!m_pDasherView || !m_pDasherModel)
     return;
   
@@ -556,17 +619,7 @@ void CDasherInterfaceBase::ChangeAlphabet() {
 
   SetBoolParameter( BP_TRAINING, true );
 
-//   m_AlphInfo = m_AlphIO->GetInfo(NewAlphabetID);
-
-//   //AlphabetID = m_AlphInfo.AlphID.c_str();
-
-//   //  std::auto_ptr < CAlphabet > ptrOld(m_Alphabet);       // So we can delete the old alphabet later
-
-//   m_Alphabet = new CAlphabet(m_AlphInfo);
-
-  delete m_pDasherModel;
-  m_pDasherModel = 0;
-  CreateDasherModel();
+  CreateNCManager();
 
   // Let our user log object know about the new alphabet since
   // it needs to convert symbols into text for the log file.
@@ -594,13 +647,12 @@ void CDasherInterfaceBase::ChangeScreen(CDasherScreen *NewScreen) {
 
   if(m_pDasherView != 0) {
     m_pDasherView->ChangeScreen(m_DasherScreen);
-  } else {
-    if(GetLongParameter(LP_VIEW_ID) != -1)
-      ChangeView();
+  } else if(GetLongParameter(LP_VIEW_ID) != -1) {
+    ChangeView();
   }
 
   PositionActionButtons();
-  ScheduleRedraw();
+  Redraw(true);
 }
 
 void CDasherInterfaceBase::ChangeView() {
@@ -616,10 +668,6 @@ void CDasherInterfaceBase::ChangeView() {
   }
 }
 
-void CDasherInterfaceBase::GetAlphabets(std::vector <std::string >*AlphabetList) {
-  m_AlphIO->GetAlphabets(AlphabetList);
-}
-
 const CAlphIO::AlphInfo & CDasherInterfaceBase::GetInfo(const std::string &AlphID) {
   return m_AlphIO->GetInfo(AlphID);
 }
@@ -632,17 +680,13 @@ void CDasherInterfaceBase::DeleteAlphabet(const std::string &AlphID) {
   m_AlphIO->Delete(AlphID);
 }
 
-void CDasherInterfaceBase::GetColours(std::vector <std::string >*ColourList) {
-  m_ColourIO->GetColours(ColourList);
-}
-
 /*
 	I've used C style I/O because I found that C++ style I/O bloated
 	the Win32 code enormously. The overhead of loading the buffer into
 	the string instead of reading straight into a string seems to be
 	negligible compared to huge requirements elsewhere.
 */
-int CDasherInterfaceBase::TrainFile(string Filename, int iTotalBytes, int iOffset) {
+int CDasherInterfaceBase::TrainFile(std::string Filename, int iTotalBytes, int iOffset) {
   if(Filename == "")
     return 0;
   
@@ -660,7 +704,7 @@ int CDasherInterfaceBase::TrainFile(string Filename, int iTotalBytes, int iOffse
 
   vector < symbol > Symbols;
 
-  CAlphabetManagerFactory::CTrainer * pTrainer = m_pDasherModel->GetTrainer();
+  CAlphabetManagerFactory::CTrainer * pTrainer = m_pNCManager->GetTrainer();
   do {
     NumberRead = fread(InputBuffer, 1, BufferSize - 1, InputFile);
     InputBuffer[NumberRead] = '\0';
@@ -734,58 +778,71 @@ void CDasherInterfaceBase::ResetNats() {
 }
 
 
-void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
-  m_pDasherModel->m_strContextBuffer = "";
+// TODO: Check that none of this needs to be reimplemented
 
-  Dasher::CEditContextEvent oEvent(10);
+// void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
+//   m_pDasherModel->m_strContextBuffer = "";
+
+//   Dasher::CEditContextEvent oEvent(10);
+//   m_pEventHandler->InsertEvent(&oEvent);
+
+//    std::string strNewContext(m_pDasherModel->m_strContextBuffer);
+
+//   // We keep track of an internal context and compare that to what
+//   // we are given - don't restart Dasher if nothing has changed.
+//   // This should really be integrated with DasherModel, which
+//   // probably will be the case when we start to deal with being able
+//   // to back off indefinitely. For now though we'll keep it in a
+//   // separate string.
+
+//    int iContextLength( 6 ); // The 'important' context length - should really get from language model
+
+//    // FIXME - use unicode lengths
+
+//    if(bForceStart || (strNewContext.substr( std::max(static_cast<int>(strNewContext.size()) - iContextLength, 0)) != strCurrentContext.substr( std::max(static_cast<int>(strCurrentContext.size()) - iContextLength, 0)))) {
+
+//      if(m_pDasherModel != NULL) {
+//        // TODO: Reimplement this
+//        //       if(m_pDasherModel->m_bContextSensitive || bForceStart) {
+//        {
+//  	m_pDasherModel->SetContext(strNewContext);
+//  	PauseAt(0,0);
+//        }
+//      }
+    
+//      strCurrentContext = strNewContext;
+//      WriteTrainFileFull();
+//    }
+
+//    if(bForceStart) {
+//      int iMinWidth;
+
+//      if(m_pInputFilter && m_pInputFilter->GetMinWidth(iMinWidth)) {
+//        m_pDasherModel->LimitRoot(iMinWidth);
+//      }
+//    }
+
+//    if(m_pDasherView)
+//      while( m_pDasherModel->CheckForNewRoot(m_pDasherView) ) {
+//        // Do nothing
+//      }
+
+//    ScheduleRedraw();
+// }
+
+// TODO: Fix this
+
+std::string CDasherInterfaceBase::GetContext(int iStart, int iLength) {
+  m_strContext = "";
+  
+  CEditContextEvent oEvent(iStart, iLength);
   m_pEventHandler->InsertEvent(&oEvent);
 
-   std::string strNewContext(m_pDasherModel->m_strContextBuffer);
-
-  // We keep track of an internal context and compare that to what
-  // we are given - don't restart Dasher if nothing has changed.
-  // This should really be integrated with DasherModel, which
-  // probably will be the case when we start to deal with being able
-  // to back off indefinitely. For now though we'll keep it in a
-  // separate string.
-
-   int iContextLength( 6 ); // The 'important' context length - should really get from language model
-
-   // FIXME - use unicode lengths
-
-   if(bForceStart || (strNewContext.substr( std::max(static_cast<int>(strNewContext.size()) - iContextLength, 0)) != strCurrentContext.substr( std::max(static_cast<int>(strCurrentContext.size()) - iContextLength, 0)))) {
-
-     if(m_pDasherModel != NULL) {
-       // TODO: Reimplement this
-       //       if(m_pDasherModel->m_bContextSensitive || bForceStart) {
-       {
- 	m_pDasherModel->SetContext(strNewContext);
- 	PauseAt(0,0);
-       }
-     }
-    
-     strCurrentContext = strNewContext;
-     WriteTrainFileFull();
-   }
-
-   if(bForceStart) {
-     int iMinWidth;
-
-     if(m_pInputFilter && m_pInputFilter->GetMinWidth(iMinWidth)) {
-       m_pDasherModel->LimitRoot(iMinWidth);
-     }
-   }
-
-   if(m_pDasherView)
-     while( m_pDasherModel->CheckForNewRoot(m_pDasherView) ) {
-       // Do nothing
-     }
-
-   ScheduleRedraw();
+  return m_strContext;
 }
 
 void CDasherInterfaceBase::SetContext(std::string strNewContext) {
-  m_pDasherModel->m_strContextBuffer = strNewContext;
+  m_strContext = strNewContext;
 }
 
 // Control mode stuff
@@ -888,7 +945,7 @@ void CDasherInterfaceBase::RegisterFactory(CModuleFactory *pFactory) {
   m_oModuleManager.RegisterFactory(pFactory);
 }
  
-CDasherModule *CDasherInterfaceBase::GetModule(long long int iID) {
+CDasherModule *CDasherInterfaceBase::GetModule(ModuleID_t iID) {
    return m_oModuleManager.GetModule(iID);
 }
 
@@ -897,19 +954,19 @@ CDasherModule *CDasherInterfaceBase::GetModuleByName(const std::string &strName)
 }
 
 void CDasherInterfaceBase::CreateFactories() {
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDefaultFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel,3, "Normal Control")));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDefaultFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel,3, _("Normal Control"))));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new COneDimensionalFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CEyetrackerFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CClickFilter(m_pEventHandler, m_pSettingsStore, this)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDynamicFilter(m_pEventHandler, m_pSettingsStore, this)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this, 14, 1, "Two Button Dynamic Mode")));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this, 14, 1, _("Two Button Dynamic Mode"))));
   // TODO: specialist factory for button mode
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 5, 1, true,8, "Menu Mode")));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 0, false,10, "Direct Mode")));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 5, 1, true,8, _("Menu Mode"))));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 0, false,10, _("Direct Mode"))));
   //  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 4, 0, false,11, "Buttons 3")));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 3, false,12, "Alternating Direct Mode")));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 4, 2, false,13, "Compass Mode")));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CStylusFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel,15, "Stylus Control")));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 3, false,12, _("Alternating Direct Mode"))));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 4, 2, false,13, _("Compass Mode"))));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CStylusFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel,15, _("Stylus Control"))));
 
 }
 
@@ -917,10 +974,12 @@ void CDasherInterfaceBase::GetPermittedValues(int iParameter, std::vector<std::s
   // TODO: Deprecate direct calls to these functions
   switch (iParameter) {
   case SP_ALPHABET_ID:
-    GetAlphabets(&vList);
+    if(m_AlphIO)
+      m_AlphIO->GetAlphabets(&vList);
     break;
   case SP_COLOUR_ID:
-    GetColours(&vList);
+    if(m_ColourIO)
+      m_ColourIO->GetColours(&vList);
     break;
   case SP_INPUT_FILTER:
     m_oModuleManager.ListModules(1, vList);
@@ -1108,3 +1167,30 @@ void CDasherInterfaceBase::ReleaseLock(int iLockFlags) {
   if(m_iLockCount == 0)
     ChangeState(TR_UNLOCK);
 }
+
+void CDasherInterfaceBase::SetBuffer(int iOffset) { 
+   CreateModel(iOffset);
+}
+
+void CDasherInterfaceBase::UnsetBuffer() {
+  // TODO: Write training file?
+  if(m_pDasherModel)
+    delete m_pDasherModel;
+
+  m_pDasherModel = 0;
+}
+
+void CDasherInterfaceBase::SetOffset(int iOffset) {
+  if(m_pDasherModel)
+    m_pDasherModel->SetOffset(iOffset, m_pDasherView);
+}
+
+void CDasherInterfaceBase::SetControlOffset(int iOffset) {
+  if(m_pDasherModel)
+    m_pDasherModel->SetControlOffset(iOffset);
+}
+
+void CDasherInterfaceBase::ClSet(const std::string &strKey, const std::string &strValue) {
+  if(m_pSettingsStore)
+    m_pSettingsStore->ClSet(strKey, strValue);
+}
