@@ -11,9 +11,10 @@
 #include "PPMLanguageModel.h"
 
 #include <cmath>
-#include <stack>
-#include <iostream>
+#include <cstdlib>
 #include <fstream>
+#include <iostream>
+#include <stack>
 
 using namespace Dasher;
 using namespace std;
@@ -27,7 +28,7 @@ typedef unsigned long ulong;
 
 // Track memory leaks on Windows to the line that new'd the memory
 #ifdef _WIN32
-#ifdef _DEBUG
+#ifdef _DEBUG_MEMLEAKS
 #define DEBUG_NEW new( _NORMAL_BLOCK, THIS_FILE, __LINE__ )
 #define new DEBUG_NEW
 #undef THIS_FILE
