@@ -1,4 +1,3 @@
-
 #include "../Common/Common.h"
 
 #include "UserLog.h"
@@ -12,7 +11,7 @@
 
 // Track memory leaks on Windows to the line that new'd the memory
 #ifdef _WIN32
-#ifdef _DEBUG
+#ifdef _DEBUG_MEMLEAKS
 #define DEBUG_NEW new( _NORMAL_BLOCK, THIS_FILE, __LINE__ )
 #define new DEBUG_NEW
 #undef THIS_FILE
@@ -31,8 +30,6 @@ static UserLogParamMask s_UserLogParamMaskTable [] = {
   {BP_CONTROL_MODE,         userLogParamOutputToSimple},
   {LP_UNIFORM,              userLogParamOutputToSimple},
   {LP_YSCALE,               userLogParamOutputToSimple},
-  {BP_NUMBER_DIMENSIONS,    userLogParamOutputToSimple},
-  {BP_EYETRACKER_MODE,      userLogParamOutputToSimple},
   {LP_LANGUAGE_MODEL_ID,    userLogParamOutputToSimple},
   {LP_LM_MAX_ORDER,         userLogParamOutputToSimple},
   {LP_LM_EXCLUSION,         userLogParamOutputToSimple},
@@ -47,7 +44,7 @@ static UserLogParamMask s_UserLogParamMaskTable [] = {
 CUserLog::CUserLog(Dasher::CEventHandler *pEventHandler, 
                    CSettingsStore *pSettingsStore, 
                    int iLogTypeMask, 
-                   Dasher::CAlphabet* pAlphabet) : CDasherComponent(pEventHandler, pSettingsStore)
+                   Dasher::CAlphabet* pAlphabet) : CUserLogBase(pEventHandler, pSettingsStore)
 {
   //CFunctionLogger f1("CUserLog::CUserLog", g_pLogger);
 
@@ -256,7 +253,7 @@ void CUserLog::StopWriting()
         return;
       }
 
-      pTrial->StopWriting();
+      pTrial->StopWriting(GetCycleBits());
     }
   }
 }
@@ -698,6 +695,13 @@ int CUserLog::GetLogLevelMask()
 
   return m_iLevelMask;
 }
+
+void CUserLog::KeyDown(int iId, int iType, int iEffect) {
+  CUserLogTrial* pTrial = GetCurrentTrial();
+  
+  if(pTrial)
+    pTrial->AddKeyDown(iId, iType, iEffect);
+}
   
 // This gets called whenever parameters get changed that we are tracking
 void CUserLog::HandleEvent(Dasher::CEvent* pEvent)
@@ -815,7 +819,7 @@ string CUserLog::GetXML()
   {
     CUserLogTrial* pTrial = (CUserLogTrial*) m_vpTrials[i];
 
-    // Only log trials that actually had some writing in it
+   // Only log trials that actually had some writing in it
     if ((pTrial != NULL) && (pTrial->HasWritingOccured()))
     {
       strResult += pTrial->GetXML("\t\t");
@@ -1023,8 +1027,8 @@ void CUserLog::PrepareNewTrial()
     VECTOR_STRING vFound;
     if (m_vParams.size() > 0)
     {
-      for (VECTOR_USER_LOG_PARAM_PTR_ITER iter = m_vParams.end() - 1; iter >= m_vParams.begin(); iter--)
-      {
+	  for (VECTOR_USER_LOG_PARAM_PTR_REV_ITER iter = m_vParams.rbegin(); iter != m_vParams.rend(); ++iter)
+	  {
         if (((*iter) != NULL) && ((*iter)->options & userLogParamForceInTrial))
         {
           // Make sure we haven't output this one already
@@ -1171,7 +1175,7 @@ void CUserLog::UpdateParam(int iParameter, int iOptionMask)
 // UserLog XML files and does cool things to them.  
 
 // Load the object from an XML file
-CUserLog::CUserLog(string strXMLFilename) : CDasherComponent(NULL, NULL)
+CUserLog::CUserLog(string strXMLFilename) : CUserLogBase(NULL, NULL)
 {
   //CFunctionLogger f1("CUserLog::CUserLog(XML)", g_pLogger);
 
