@@ -37,8 +37,8 @@ CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl)
 
   Realize();
  
-  m_pKeyboardHelper = new CKeyboardHelper(this);
-  m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
+  //  m_pKeyboardHelper = new CKeyboardHelper(this);
+  //  m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
 }
 
 void CDasherControl::CreateLocalFactories() {
@@ -51,6 +51,11 @@ void CDasherControl::CreateLocalFactories() {
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherJoystickInputDiscrete(m_pEventHandler, m_pSettingsStore, this)));
 #endif
   
+#ifdef TILT
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherTiltInput(m_pEventHandler, m_pSettingsStore, this)));
+#endif
+  
+  
   // Create locally cached copies of the mouse input objects, as we
   // need to pass coordinates to them from the timer callback
   
@@ -198,10 +203,10 @@ CDasherControl::~CDasherControl() {
     m_pPangoCache = NULL;
   }
 
-  if(m_pKeyboardHelper) {
-    delete m_pKeyboardHelper;
-    m_pKeyboardHelper = 0;
-  }
+//   if(m_pKeyboardHelper) {
+//     delete m_pKeyboardHelper;
+//     m_pKeyboardHelper = 0;
+//   }
 }
 
 bool CDasherControl::FocusEvent(GtkWidget *pWidget, GdkEventFocus *pEvent) {
@@ -290,11 +295,11 @@ void CDasherControl::ExternalEventHandler(Dasher::CEvent *pEvent) {
     
     if(pEditEvent->m_iEditType == 1) {
       // Insert event
-      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_insert", pEditEvent->m_sText.c_str());
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_insert", pEditEvent->m_sText.c_str(), pEditEvent->m_iOffset);
     }
     else if(pEditEvent->m_iEditType == 2) {
       // Delete event
-      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_delete", pEditEvent->m_sText.c_str());
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_delete", pEditEvent->m_sText.c_str(), pEditEvent->m_iOffset);
     }
     else if(pEditEvent->m_iEditType == 10) {
       g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_convert");
@@ -352,22 +357,25 @@ void CDasherControl::WriteTrainFile(const std::string &strNewText) {
   close(fd);
 }
 
+// TODO: Sort these methods out
 void CDasherControl::ExternalKeyDown(int iKeyVal) {
-  if(m_pKeyboardHelper) {
-    int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
+//   if(m_pKeyboardHelper) {
+//     int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
 
-    if(iButtonID != -1)
-      KeyDown(get_time(), iButtonID);
-  }
+//     if(iButtonID != -1)
+//       KeyDown(get_time(), iButtonID);
+//   }
+  KeyDown(get_time(), iKeyVal);
 }
 
 void CDasherControl::ExternalKeyUp(int iKeyVal) {
-  if(m_pKeyboardHelper) {
-    int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
+//   if(m_pKeyboardHelper) {
+//     int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
     
-    if(iButtonID != -1)
-      KeyUp(get_time(), iButtonID);
-  }
+//     if(iButtonID != -1)
+//       KeyUp(get_time(), iButtonID);
+//   }
+  KeyUp(get_time(), iKeyVal);
 }
 
 void CDasherControl::HandleParameterNotification(int iParameter) {
@@ -379,8 +387,9 @@ void CDasherControl::HandleParameterNotification(int iParameter) {
     }
     break;
   case BP_GLOBAL_KEYBOARD:
-    if(m_pKeyboardHelper)
-      m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
+    // TODO: reimplement
+//     if(m_pKeyboardHelper)
+//       m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
     break;
   }
 }
@@ -401,9 +410,9 @@ int CDasherControl::TimerEvent() {
   if(GetLongParameter(LP_YSCALE) < 10)
     SetLongParameter(LP_YSCALE, 10);
 
-  y = (y - iRootHeight / 2) * 4096 / GetLongParameter(LP_YSCALE);
+  y = (y - iRootHeight / 2);
 
-  m_p1DMouseInput->SetCoordinates(x, y);
+  m_p1DMouseInput->SetCoordinates(y, GetLongParameter(LP_YSCALE));
 
   NewFrame(get_time(), false);
 
@@ -434,6 +443,7 @@ int CDasherControl::TimerEvent() {
       int iMouseY = 0;  
       gdk_window_get_pointer(NULL, &iMouseX, &iMouseY, NULL);
 
+      // TODO: This sort of thing shouldn't be in specialised methods, move into base class somewhere
       pUserLog->AddMouseLocationNormalized(iMouseX, iMouseY, true, GetNats());
   }
 
@@ -457,16 +467,18 @@ gboolean CDasherControl::ButtonPressEvent(GdkEventButton *event) {
 
   // Take the focus if we click on the canvas
 
-  GdkEventFocus *focusEvent = (GdkEventFocus *) g_malloc(sizeof(GdkEventFocus));
-  gboolean *returnType;
+//   GdkEventFocus *focusEvent = (GdkEventFocus *) g_malloc(sizeof(GdkEventFocus));
+//   gboolean *returnType;
 
-  focusEvent->type = GDK_FOCUS_CHANGE;
-  focusEvent->window = (GdkWindow *) m_pCanvas;
-  focusEvent->send_event = FALSE;
-  focusEvent->in = TRUE;
+//   focusEvent->type = GDK_FOCUS_CHANGE;
+//   focusEvent->window = (GdkWindow *) m_pCanvas;
+//   focusEvent->send_event = FALSE;
+//   focusEvent->in = TRUE;
 
-  gtk_widget_grab_focus(GTK_WIDGET(m_pCanvas));
-  g_signal_emit_by_name(GTK_OBJECT(m_pCanvas), "focus_in_event", GTK_WIDGET(m_pCanvas), focusEvent, NULL, &returnType);
+//   gtk_widget_grab_focus(GTK_WIDGET(m_pCanvas));
+//   g_signal_emit_by_name(GTK_OBJECT(m_pCanvas), "focus_in_event", GTK_WIDGET(m_pCanvas), focusEvent, NULL, &returnType);
+
+  // No - don't take the focus - give it to the text area instead
   
   if(event->type == GDK_BUTTON_PRESS)
     HandleClickDown(get_time(), (int)event->x, (int)event->y);
@@ -494,12 +506,12 @@ gint CDasherControl::KeyReleaseEvent(GdkEventKey *event) {
 //       SetLongParameter(LP_BOOSTFACTOR, 100);
   }
   else {
-    if(m_pKeyboardHelper) {
-      int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
+//     if(m_pKeyboardHelper) {
+//       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
-      if(iKeyVal != -1)
-	KeyUp(get_time(), iKeyVal);
-    }
+//       if(iKeyVal != -1)
+// 	KeyUp(get_time(), iKeyVal);
+//     }
   }
 
   return 0;
@@ -511,12 +523,12 @@ gint CDasherControl::KeyPressEvent(GdkEventKey *event) {
   //  else if((event->keyval == GDK_Control_L) || (event->keyval == GDK_Control_R))
   //   SetLongParameter(LP_BOOSTFACTOR, 25);
   // else {
-    if(m_pKeyboardHelper) {
-      int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
+//     if(m_pKeyboardHelper) {
+//       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
-      if(iKeyVal != -1)
-	KeyDown(get_time(), iKeyVal);
-    }
+//       if(iKeyVal != -1)
+// 	KeyDown(get_time(), iKeyVal);
+//     }
     //  }
   return 0;
 }
