@@ -14,15 +14,6 @@
 typedef struct _DasherMain DasherMain;
 struct _DasherMain;
 
-typedef enum {
-  CLIPBOARD_CUT,
-  CLIPBOARD_COPY,
-  CLIPBOARD_PASTE,
-  CLIPBOARD_COPYALL,
-  CLIPBOARD_SELECTALL,
-  CLIPBOARD_CLEAR,
-} clipboard_action;
-
 G_BEGIN_DECLS
 #define TYPE_DASHER_EDITOR            (dasher_editor_get_type())
 #define DASHER_EDITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), TYPE_DASHER_EDITOR, DasherEditor ))
@@ -35,18 +26,43 @@ typedef struct _DasherEditor DasherEditor;
 typedef struct _DasherEditorClass DasherEditorClass;
 
 struct _DasherEditor {
-  GObject parent;
+  GtkVBox box;
 };
 
 struct _DasherEditorClass {
-  GObjectClass parent_class;
+  GtkVBoxClass parent_class;
 
   void (*filename_changed)(DasherEditor *pDasherEditor);
   void (*buffer_changed)(DasherEditor *pDasherEditor);
   void (*context_changed)(DasherEditor *pDasherEditor);
+
+
+  void (*initialise)(DasherEditor *pSelf, DasherAppSettings *pAppSettings, DasherMain *pDasherMain, GladeXML *pGladeXML, const gchar *szFullPath);
+  gboolean (*command)(DasherEditor *pSelf, const gchar *szCommand);
+  void (*handle_font)(DasherEditor *pSelf, const gchar *szFont);
+  void (*output)(DasherEditor *pSelf, const gchar *szText, int iOffset);
+  void (*delete_text)(DasherEditor *pSelf, int iLength, int iOffset);
+  const gchar *(*get_context)(DasherEditor *pSelf, int iOffset, int iLength);
+  gint (*get_offset)(DasherEditor *pSelf);
+  void (*handle_stop)(DasherEditor *pSelf);
+  void (*handle_start)(DasherEditor *pSelf);
+  void (*handle_control)(DasherEditor *pSelf, int iNodeID);
+  void (*action_button)(DasherEditor *pSelf, DasherAction *pAction);
+  void (*actions_start)(DasherEditor *pSelf);
+  bool (*actions_more)(DasherEditor *pSelf);
+  void (*actions_get_next)(DasherEditor *pSelf, const gchar **szName, gint *iID, gboolean *bShow, gboolean *bControl, gboolean *bAuto);
+  void (*action_set_show)(DasherEditor *pSelf, int iActionID, bool bValue);
+  void (*action_set_control)(DasherEditor *pSelf, int iActionID, bool bValue);
+  void (*action_set_auto)(DasherEditor *pSelf, int iActionID, bool bValue);
+  void (*grab_focus)(DasherEditor *pSelf);
+  gboolean (*file_changed)(DasherEditor *pSelf);
+  const gchar *(*get_filename)(DasherEditor *pSelf);
+  const gchar *(*get_all_text)(DasherEditor *pSelf);
+  const gchar *(*get_new_text)(DasherEditor *pSelf);
 };
 
-DasherEditor *dasher_editor_new(DasherAppSettings *pAppSettings, DasherMain *pDasherMain, GladeXML *pGladeXML, const gchar *szFullPath);
+void dasher_editor_initialise(DasherEditor *pSelf, DasherAppSettings *pAppSettings, DasherMain *pDasherMain, GladeXML *pGladeXML, const gchar *szFullPath);
+
 GType dasher_editor_get_type();
 
 /* Replace this with GTK text buffer */
@@ -55,9 +71,11 @@ GType dasher_editor_get_type();
 // TODO: Just have one 'handle event' method?
 gboolean dasher_editor_command(DasherEditor *pSelf, const gchar *szCommand);
 
+void dasher_editor_handle_font(DasherEditor *pSelf, const gchar *szFont);
+
 /* To be obsoleted by movement to GTK buffers */
-void dasher_editor_output(DasherEditor *pSelf, const gchar *szText);
-void dasher_editor_delete(DasherEditor *pSelf, int iLength);
+void dasher_editor_output(DasherEditor *pSelf, const gchar *szText, int iOffset);
+void dasher_editor_delete(DasherEditor *pSelf, int iLength, int iOffset);
 const gchar *dasher_editor_get_context(DasherEditor *pSelf, int iOffset, int iLength);
 gint dasher_editor_get_offset(DasherEditor *pSelf);
 
