@@ -1,6 +1,8 @@
 #include <gtk/gtk.h>
+#include <iostream>
 #include <string.h>
 
+#include "../Common/Common.h"
 #include "dasher_buffer_set.h"
 #include "dasher_internal_buffer.h"
 
@@ -8,14 +10,15 @@ static void dasher_internal_buffer_class_init(DasherInternalBufferClass *pClass)
 static void dasher_internal_buffer_init(DasherInternalBuffer *pAction);
 static void dasher_internal_buffer_destroy(GObject *pObject);
 static void idasher_buffer_set_interface_init (gpointer g_iface, gpointer iface_data);
-void dasher_internal_buffer_insert(DasherInternalBuffer *pSelf, const gchar *szText);
-void dasher_internal_buffer_delete(DasherInternalBuffer *pSelf, int iLength);
-gchar *dasher_internal_buffer_get_context(DasherInternalBuffer *pSelf, gint iMaxLength);
+void dasher_internal_buffer_insert(DasherInternalBuffer *pSelf, const gchar *szText, int iOffset);
+void dasher_internal_buffer_delete(DasherInternalBuffer *pSelf, int iLength, int iOffset);
+gchar *dasher_internal_buffer_get_context(DasherInternalBuffer *pSelf, gint iOffset, gint iLength);
 void dasher_internal_buffer_edit_move(DasherInternalBuffer *pSelf, int iDirection, int iDist);
 void dasher_internal_buffer_edit_delete(DasherInternalBuffer *pSelf, int iDirection, int iDist);
 void dasher_internal_buffer_edit_convert(DasherInternalBuffer *pSelf);
 void dasher_internal_buffer_edit_protect(DasherInternalBuffer *pSelf);
 void dasher_internal_buffer_conversion_mode(DasherInternalBuffer *pSelf, gboolean bMode);
+gint dasher_internal_buffer_get_offset(DasherInternalBuffer *pSelf);
 
 // Related signal handlers
 extern "C" void mark_set_handler(GtkWidget *widget, GtkTextIter *pIter, GtkTextMark *pMark, gpointer pUserData);
@@ -78,14 +81,15 @@ static void dasher_internal_buffer_init(DasherInternalBuffer *pDasherControl) {
 
 static void idasher_buffer_set_interface_init (gpointer g_iface, gpointer iface_data) {
   IDasherBufferSetInterface *iface = (IDasherBufferSetInterface *)g_iface;
-  iface->insert_text = (void (*)(IDasherBufferSet *pSelf, const gchar *szText))dasher_internal_buffer_insert;
-  iface->delete_text = (void (*)(IDasherBufferSet *pSelf, gint iLength))dasher_internal_buffer_delete;
-  iface->get_context = (gchar *(*)(IDasherBufferSet *pSelf, gint iMaxLength))dasher_internal_buffer_get_context;
+  iface->insert_text = (void (*)(IDasherBufferSet *pSelf, const gchar *szText, int iOffset))dasher_internal_buffer_insert;
+  iface->delete_text = (void (*)(IDasherBufferSet *pSelf, gint iLength, int iOffset))dasher_internal_buffer_delete;
+  iface->get_context = (gchar *(*)(IDasherBufferSet *pSelf, gint iOffset, gint iLength))dasher_internal_buffer_get_context;
   iface->edit_move = (void (*)(IDasherBufferSet *pSelf, gint iDirection, gint iDist))dasher_internal_buffer_edit_move;
   iface->edit_delete = (void (*)(IDasherBufferSet *pSelf, gint iDirection, gint iDist))dasher_internal_buffer_edit_delete;
   iface->edit_convert = (void (*)(IDasherBufferSet *pSelf))dasher_internal_buffer_edit_convert;
   iface->edit_protect = (void (*)(IDasherBufferSet *pSelf))dasher_internal_buffer_edit_protect;
   iface->conversion_mode = (void (*)(IDasherBufferSet *pSelf, gboolean bMode))dasher_internal_buffer_conversion_mode;
+  iface->get_offset = (gint (*)(IDasherBufferSet *pSelf))dasher_internal_buffer_get_offset;
 }
 
 static void dasher_internal_buffer_destroy(GObject *pObject) {
@@ -104,7 +108,14 @@ DasherInternalBuffer *dasher_internal_buffer_new(GtkTextView *pTextView) {
   pPrivate->pBuffer = gtk_text_view_get_buffer(pTextView);
 
   pPrivate->pOutputTag = gtk_text_buffer_create_tag(pPrivate->pBuffer, NULL, NULL);
+
+#if GTK_CHECK_VERSION(2,8,0)
   pPrivate->pHiddenTag = gtk_text_buffer_create_tag(pPrivate->pBuffer, NULL, "invisible", TRUE, NULL);
+#else
+  // TODO: We really ought to do something a little more sensible with conversion in GTK < 2.8
+  pPrivate->pHiddenTag = gtk_text_buffer_create_tag(pPrivate->pBuffer, NULL, NULL);
+#endif
+
   pPrivate->pVisibleTag = gtk_text_buffer_create_tag(pPrivate->pBuffer, NULL, "foreground", "red", NULL);
 
   pPrivate->bConversionMode = FALSE;
@@ -116,15 +127,19 @@ DasherInternalBuffer *dasher_internal_buffer_new(GtkTextView *pTextView) {
   return pDasherControl;
 }
 
-void dasher_internal_buffer_insert(DasherInternalBuffer *pSelf, const gchar *szText) { 
+void dasher_internal_buffer_insert(DasherInternalBuffer *pSelf, const gchar *szText, int iOffset) { 
   DasherInternalBufferPrivate *pPrivate = (DasherInternalBufferPrivate *)(pSelf->private_data);
 
+  //  std::cout << "i: " << szText << " (" << iOffset << " " << gtk_text_buffer_get_char_count(pPrivate->pBuffer) << ")" << std::endl;
+
+  DASHER_ASSERT(gtk_text_buffer_get_char_count(pPrivate->pBuffer) == iOffset);
+
   gtk_text_buffer_delete_selection(pPrivate->pBuffer, false, true );
 
   GtkTextIter sIter;
   gtk_text_buffer_get_iter_at_mark(pPrivate->pBuffer, &sIter, gtk_text_buffer_get_insert(pPrivate->pBuffer));
 
-  GtkTextTag *pCurrentTag;
+  GtkTextTag *pCurrentTag = NULL;
 
   if(!pPrivate->bConversionMode)
     pCurrentTag = pPrivate->pOutputTag;
@@ -139,13 +154,17 @@ void dasher_internal_buffer_insert(DasherInternalBuffer *pSelf, const gchar *szT
     }
   }
 
+  if(!pCurrentTag)
+    return;
+
   gtk_text_buffer_insert_with_tags(pPrivate->pBuffer, &sIter, szText, -1, pCurrentTag, NULL);
 
   gtk_text_view_scroll_mark_onscreen(pPrivate->pTextView, gtk_text_buffer_get_insert(pPrivate->pBuffer));
 
+  //  g_message("Buffer lenght: %d", gtk_text_buffer_get_char_count(pPrivate->pBuffer));
 }
 
-void dasher_internal_buffer_delete(DasherInternalBuffer *pSelf, int iLength) { 
+void dasher_internal_buffer_delete(DasherInternalBuffer *pSelf, int iLength, int iOffset) { 
   DasherInternalBufferPrivate *pPrivate = (DasherInternalBufferPrivate *)(pSelf->private_data);
   
   GtkTextIter *start = new GtkTextIter;
@@ -163,25 +182,30 @@ void dasher_internal_buffer_delete(DasherInternalBuffer *pSelf, int iLength) {
 
   delete start;
   delete end;
+
+  //  std::cout << "d: " << iLength << " (" << iOffset << " " << gtk_text_buffer_get_char_count(pPrivate->pBuffer) << ")" << std::endl; 
+  DASHER_ASSERT(gtk_text_buffer_get_char_count(pPrivate->pBuffer) == iOffset);
 }
 
-gchar *dasher_internal_buffer_get_context(DasherInternalBuffer *pSelf, gint iMaxLength) {
+gchar *dasher_internal_buffer_get_context(DasherInternalBuffer *pSelf, gint iOffset, gint iLength) {
   DasherInternalBufferPrivate *pPrivate = (DasherInternalBufferPrivate *)(pSelf->private_data);
 
+  //  g_message("Buffer lenght: %d", gtk_text_buffer_get_char_count(pPrivate->pBuffer));
+
   GtkTextIter start;
   GtkTextIter end; // Refers to end of context, which is start of selection!
 
-  gtk_text_buffer_get_selection_bounds( pPrivate->pBuffer, &end, NULL );
-  start = end;
-  gtk_text_iter_backward_chars( &start, iMaxLength );
+  gtk_text_buffer_get_iter_at_offset(pPrivate->pBuffer, &start, iOffset);
+  gtk_text_buffer_get_iter_at_offset(pPrivate->pBuffer, &end, iOffset + iLength);
 
   return gtk_text_buffer_get_text( pPrivate->pBuffer, &start, &end, false );
 }
 
+// TODO: Check that these signals are being used appropriately
 void dasher_internal_buffer_change_context(DasherInternalBuffer *pSelf, GtkTextIter *pIter, GtkTextMark *pMark) {
   const char *szMarkName(gtk_text_mark_get_name(pMark));
   if(szMarkName && !strcmp(szMarkName,"insert")) {
-    g_signal_emit(pSelf, idasher_buffer_set_signals[CONTEXT_CHANGED], 0, NULL);
+    g_signal_emit_by_name(G_OBJECT(pSelf), "offset_changed", G_OBJECT(pSelf), NULL, NULL);
   }
 }
 
@@ -339,12 +363,22 @@ void dasher_internal_buffer_clear(DasherInternalBuffer *pSelf) {
 
   gtk_text_buffer_delete(pPrivate->pBuffer, start, end);
 
-  g_signal_emit(pSelf, idasher_buffer_set_signals[CONTEXT_CHANGED], 0, NULL);
+  /* TODO: this probably shouldn't emit a signal */
+  g_signal_emit_by_name(G_OBJECT(pSelf), "buffer_changed", G_OBJECT(pSelf), NULL, NULL);
 
   pPrivate->iCurrentState = 0;
   pPrivate->iLastOffset = 0;
 }
 
+gint dasher_internal_buffer_get_offset(DasherInternalBuffer *pSelf) {
+  DasherInternalBufferPrivate *pPrivate = (DasherInternalBufferPrivate *)(pSelf->private_data);
+ 
+  GtkTextIter oIter;
+  gtk_text_buffer_get_iter_at_mark(pPrivate->pBuffer, &oIter, gtk_text_buffer_get_mark(pPrivate->pBuffer, "insert"));
+  return gtk_text_iter_get_offset(&oIter);
+}
+
+
 // Handlers
 
 extern "C" void mark_set_handler(GtkWidget *widget, GtkTextIter *pIter, GtkTextMark *pMark, gpointer pUserData) {
