@@ -19,6 +19,10 @@
 #endif
 #include "dasher_main.h"
 
+#include "DasherAppSettings.h"
+#include "dasher_editor_internal.h"
+#include "dasher_editor_external.h"
+
 /* Static instance of singleton, USE SPARINGLY */
 static DasherMain *g_pDasherMain = NULL; 
 
@@ -33,7 +37,7 @@ struct _DasherMainPrivate {
   // Child objects owned here
   DasherAppSettings *pAppSettings;
   DasherPreferencesDialogue *pPreferencesDialogue;
-  DasherEditorInternal *pEditor;
+  DasherEditor *pEditor;
 
   CKeyboardHelper *pKeyboardHelper;
 
@@ -113,7 +117,10 @@ static DasherMenuCommand MenuCommands[] = {
   {NULL, "tb_command_saveas", "saveas"},
   {NULL, "tb_command_cut", "cut"},
   {NULL, "tb_command_copy", "copy"},
-  {NULL, "tb_command_paste", "paste"}
+  {NULL, "tb_command_paste", "paste"},
+  {NULL, "tb_command_preferences", "preferences"},
+  {NULL, "tb_command_help", "help"},
+  {NULL, "tb_command_quit", "quit"}
 };
 
 G_DEFINE_TYPE(DasherMain, dasher_main, G_TYPE_OBJECT);
@@ -169,9 +176,9 @@ extern "C" GtkWidget *create_dasher_editor(gchar *szName, gchar *szString1, gcha
 extern "C" gboolean dasher_main_cb_menu_command(GtkWidget *pWidget, gpointer pUserData);
 extern "C" gboolean speed_changed(GtkWidget *pWidget, gpointer user_data);
 extern "C" void alphabet_combo_changed(GtkWidget *pWidget, gpointer pUserData);
-extern "C" void dasher_main_cb_filename_changed(DasherEditorInternal *pEditor, gpointer pUserData);
-extern "C" void dasher_main_cb_buffer_changed(DasherEditorInternal *pEditor, gpointer pUserData);
-extern "C" void dasher_main_cb_context_changed(DasherEditorInternal *pEditor, gpointer pUserData);
+extern "C" void dasher_main_cb_filename_changed(DasherEditor *pEditor, gpointer pUserData);
+extern "C" void dasher_main_cb_buffer_changed(DasherEditor *pEditor, gpointer pUserData);
+extern "C" void dasher_main_cb_context_changed(DasherEditor *pEditor, gpointer pUserData);
 extern "C" gboolean dasher_main_cb_window_close(GtkWidget *pWidget, gpointer pUserData);
 extern "C" void parameter_notification(GtkDasherControl *pDasherControl, gint iParameter, gpointer data);
 
@@ -320,7 +327,7 @@ dasher_main_new(int *argc, char ***argv, SCommandLine *pCommandLine) {
 
 
 
-    dasher_editor_internal_initialise(pPrivate->pEditor, pPrivate->pAppSettings, pDasherMain, pPrivate->pGladeXML, NULL);
+    dasher_editor_initialise(pPrivate->pEditor, pPrivate->pAppSettings, pDasherMain, pPrivate->pGladeXML, NULL);
 
 
     dasher_main_setup_window(pDasherMain);
@@ -557,7 +564,7 @@ dasher_main_load_interface(DasherMain *pSelf) {
   dasher_main_connect_menus(pSelf);
 
 
-  pPrivate->pEditor = DASHER_EDITOR_INTERNAL(glade_xml_get_widget(pPrivate->pGladeXML, "DasherEditor"));
+  pPrivate->pEditor = DASHER_EDITOR(glade_xml_get_widget(pPrivate->pGladeXML, "DasherEditor"));
   // TODO: szFullPath
   pPrivate->bWidgetsInitialised = true;
 }
@@ -568,7 +575,7 @@ dasher_main_create_preferences(DasherMain *pSelf) {
   pPrivate->pPreferencesDialogue = dasher_preferences_dialogue_new(pPrivate->pGladeXML, pPrivate->pEditor, pPrivate->pAppSettings, GTK_WINDOW(pPrivate->pMainWindow));
 }
 
-// DasherEditorInternal *
+// DasherEditor *
 // dasher_main_get_editor(DasherMain *pSelf) {
 //   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
 //   return pPrivate->pEditor;
@@ -824,7 +831,7 @@ dasher_main_command(DasherMain *pSelf, const gchar *szCommand) {
   }
 
   if(pPrivate->pEditor)
-    return dasher_editor_internal_command(pPrivate->pEditor, szCommand);
+    return dasher_editor_command(pPrivate->pEditor, szCommand);
 
   return FALSE;
 }
@@ -897,7 +904,7 @@ static void
 dasher_main_refresh_font(DasherMain *pSelf) {
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
 
-  dasher_editor_internal_handle_font(pPrivate->pEditor, 
+  dasher_editor_handle_font(pPrivate->pEditor, 
 			    dasher_app_settings_get_string(pPrivate->pAppSettings, APP_SP_EDIT_FONT));
 }
 
@@ -906,7 +913,7 @@ static void
 dasher_main_set_filename(DasherMain *pSelf) {
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
 
-  const gchar *szFilename = dasher_editor_internal_get_filename(pPrivate->pEditor);
+  const gchar *szFilename = dasher_editor_get_filename(pPrivate->pEditor);
 
   if(szFilename == 0) {
     gtk_window_set_title(GTK_WINDOW(pPrivate->pMainWindow), "Dasher");
@@ -987,8 +994,8 @@ static void dasher_main_command_quit(DasherMain *pSelf) {
 
   GtkWidget *pDialogue = NULL;
 
-  if(dasher_editor_internal_file_changed(pPrivate->pEditor)) {
-    const gchar *szFilename = dasher_editor_internal_get_filename(pPrivate->pEditor);
+  if(dasher_editor_file_changed(pPrivate->pEditor)) {
+    const gchar *szFilename = dasher_editor_get_filename(pPrivate->pEditor);
 
     if(szFilename) {
       pDialogue = gtk_message_dialog_new(GTK_WINDOW(pPrivate->pMainWindow), GTK_DIALOG_MODAL, 
@@ -1284,7 +1291,11 @@ create_dasher_control(gchar *szName, gchar *szString1, gchar *szString2, gint iI
 
 extern "C" GtkWidget *
 create_dasher_editor(gchar *szName, gchar *szString1, gchar *szString2, gint iInt1, gint iInt2) {
-  return GTK_WIDGET(dasher_editor_internal_new());
+  // TODO: Do something sensible here (this is only here for test purposes)
+  if(true)
+    return GTK_WIDGET(dasher_editor_internal_new());
+  else
+    return GTK_WIDGET(dasher_editor_external_new());
 }
 
 extern "C" gboolean 
@@ -1310,23 +1321,23 @@ alphabet_combo_changed(GtkWidget *pWidget, gpointer pUserData) {
 }
 
 extern "C" void 
-dasher_main_cb_filename_changed(DasherEditorInternal *pEditor, gpointer pUserData) {
+dasher_main_cb_filename_changed(DasherEditor *pEditor, gpointer pUserData) {
   if(g_pDasherMain)
     dasher_main_set_filename(g_pDasherMain);
 }
 
 extern "C" void 
-dasher_main_cb_buffer_changed(DasherEditorInternal *pEditor, gpointer pUserData) {
+dasher_main_cb_buffer_changed(DasherEditor *pEditor, gpointer pUserData) {
   if(!g_pDasherMain)
     return;
 
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
 
-  gtk_dasher_control_set_buffer(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), dasher_editor_internal_get_offset(pPrivate->pEditor));
+  gtk_dasher_control_set_buffer(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), dasher_editor_get_offset(pPrivate->pEditor));
 }
 
 extern "C" void 
-dasher_main_cb_context_changed(DasherEditorInternal *pEditor, gpointer pUserData) {
+dasher_main_cb_context_changed(DasherEditor *pEditor, gpointer pUserData) {
   if(!g_pDasherMain)
     return;
 
@@ -1336,7 +1347,7 @@ dasher_main_cb_context_changed(DasherEditorInternal *pEditor, gpointer pUserData
   DasherMain *pDasherMain = DASHER_MAIN(g_pDasherMain);
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pDasherMain);
 
-  gtk_dasher_control_set_offset(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), dasher_editor_internal_get_offset(pPrivate->pEditor));
+  gtk_dasher_control_set_offset(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), dasher_editor_get_offset(pPrivate->pEditor));
 }
 
 extern "C" gboolean 
@@ -1419,7 +1430,7 @@ handle_context_request(GtkDasherControl * pDasherControl, gint iOffset, gint iLe
   if(!pPrivate->pEditor || !pPrivate->pDasherWidget)
     return;
 
-  gtk_dasher_control_set_context(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), dasher_editor_internal_get_context(pPrivate->pEditor, iOffset, iLength));
+  gtk_dasher_control_set_context(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), dasher_editor_get_context(pPrivate->pEditor, iOffset, iLength));
 }
 
 extern "C" void 
@@ -1449,10 +1460,10 @@ handle_control_event(GtkDasherControl *pDasherControl, gint iEvent, gpointer dat
   g_bSend = false;
 
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
-  dasher_editor_internal_handle_control(pPrivate->pEditor, iEvent);
+  dasher_editor_handle_control(pPrivate->pEditor, iEvent);
 
   gtk_dasher_control_set_control_offset(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), 
-					dasher_editor_internal_get_offset(pPrivate->pEditor));
+					dasher_editor_get_offset(pPrivate->pEditor));
 
   g_bSend = true;
   // ---
@@ -1466,7 +1477,7 @@ handle_start_event(GtkDasherControl *pDasherControl, gpointer data) {
 
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
   
-  dasher_editor_internal_grab_focus(pPrivate->pEditor);
+  dasher_editor_grab_focus(pPrivate->pEditor);
 }
 
 
@@ -1498,3 +1509,91 @@ dasher_main_key_snooper(GtkWidget *pWidget, GdkEventKey *pEvent, gpointer pUserD
     return FALSE;
   }
 }
+
+// Callbacks from the Dasher widget
+
+extern "C" void 
+handle_stop_event(GtkDasherControl *pDasherControl, gpointer data) {
+  if(g_pDasherMain) {
+    DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+
+    if(pPrivate->pEditor)
+      dasher_editor_handle_stop(pPrivate->pEditor);
+  }
+}
+
+extern "C" void 
+on_message(GtkDasherControl *pDasherControl, gpointer pMessageInfo, gpointer pUserData) { 
+  // TODO: I don't believe that this is widely used, but possibly need to reimplement
+
+//   if(g_pDasherMain) {
+//     DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+
+//     if(pPrivate->pEditor)
+//        dasher_editor_display_message(pPrivate->pEditor, (DasherMessageInfo *)pMessageInfo);
+//   }
+}
+
+extern "C" void 
+on_command(GtkDasherControl *pDasherControl, gchar *szCommand, gpointer pUserData) {
+  if(g_pDasherMain) {
+    DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+
+    if(pPrivate->pEditor)
+      dasher_editor_command(pPrivate->pEditor, szCommand);
+  }
+}
+
+// TODO: The following two should probably be made the same
+extern "C" void 
+handle_request_settings(GtkDasherControl * pDasherControl, gpointer data) {
+  // TODO: reimplement
+  //  dasher_preferences_dialogue_show(g_pPreferencesDialogue);
+}
+
+extern "C" void 
+gtk2_edit_delete_callback(GtkDasherControl *pDasherControl, const gchar *szText, int iOffset, gpointer user_data) {
+  if(g_pDasherMain) {
+    DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+
+    if(pPrivate->pEditor) {
+      gint displaylength = g_utf8_strlen(szText, -1);
+      dasher_editor_delete(pPrivate->pEditor, displaylength, iOffset);
+    }
+  }
+}
+
+extern "C" void 
+gtk2_edit_output_callback(GtkDasherControl *pDasherControl, const gchar *szText, int iOffset, gpointer user_data) {
+  if(g_pDasherMain) {
+    DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+
+    if(pPrivate->pEditor) {
+      dasher_editor_output(pPrivate->pEditor, szText, iOffset);
+    }
+  }
+}
+
+// TODO: The following aren't exported from the editor - need to fix this
+
+extern "C" void 
+convert_cb(GtkDasherControl *pDasherControl, gpointer pUserData) {
+//   if(g_pDasherMain) {
+//     DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+
+//     if(pPrivate->pEditor) {
+//       dasher_editor_convert(pPrivate->pEditor);
+//     }
+//   }
+}
+
+extern "C" void 
+protect_cb(GtkDasherControl *pDasherControl, gpointer pUserData) {
+//   if(g_pDasherMain) {
+//     DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+
+//     if(pPrivate->pEditor) {
+//       dasher_editor_protect(pPrivate->pEditor);
+//     }
+//   }
+}
