@@ -20,13 +20,29 @@ public:
   CDasher(HWND Parent);
   ~CDasher(void);
 
+  // The following functions will not be part of the final interface
+  // In order to get rid of this, it'll be necessary to make this class
+  // act more like a UI widget.
+
+  //CCanvas *GetCanvas() {
+	 // return m_pCanvas;
+  //}
+
+  // TODO: Can some of these be obtained by inheriting from something?
+  void Move(int iX, int iY, int iWidth, int iHeight);
+  void TakeFocus();
+
   void SetEdit(CDashEditbox * pEdit);
+
   void ExternalEventHandler(Dasher::CEvent *pEvent);
+
   virtual void WriteTrainFile(const std::string &strNewText);
+
   void Main(); 
-  void Layout(int iX, int iY, int iWidth, int iHeight);
+
 
 private:
+
   virtual void ScanAlphabetFiles(std::vector<std::string> &vFileList);
   virtual void ScanColourFiles(std::vector<std::string> &vFileList);
   virtual void SetupPaths();
