@@ -12,11 +12,10 @@
 #include "resource.h"
 
 #include "Widgets/Splitter.h"
+#include "Widgets/StatusControl.h"
 #include "Widgets/Menu.h"
-//#include "Widgets/SplashScreen.h"
-//#include "Widgets/WindowSelect.h"
-#include "Widgets/Edit.h"
-
+#include "Widgets/SplashScreen.h"
+#include "Widgets/WindowSelect.h"
 #include "DasherMouseInput.h"
 #include "AppSettings.h"
 #include "GameModeHelper.h"
@@ -99,8 +98,9 @@ private:
 	// Widgets:
 	CToolbar *m_pToolbar;
 	CEdit *m_pEdit;
+	//CCanvas *m_pCanvas;
 	CSplitter *m_pSplitter;
-	CSlidebar *m_pSlidebar;
+	CStatusControl *m_pSlidebar;
 	CMenu WinMenu;
 	//CSplash *Splash;
 
