@@ -13,14 +13,15 @@
 
 #include "../resource.h"
 
-#include "../DasherInterface.h"
+#include "../../DasherCore/DasherInterfaceBase.h"
 #include "../../DasherCore/ColourIO.h"
+#include "../ModuleSettings.h"
 
 class CAppSettings;
 
 class CControlPage:public CPrefsPageBase {
 public:
-  CControlPage(HWND Parent, CDasherInterface * DI, CAppSettings *pAppSettings);
+  CControlPage(HWND Parent, CDasherInterfaceBase * DI, CAppSettings *pAppSettings);
 protected:
   LRESULT WndProc(HWND Window, UINT message, WPARAM wParam, LPARAM lParam);
 private:
@@ -38,7 +39,8 @@ private:
   
   void PopulateList();
   bool Apply();
-  bool Validate();
+ 
+  CModuleSettings *m_pModuleSettingsDialogue;
 };
 
 #endif  // ndef
