@@ -60,36 +60,24 @@
 #define IDB_COPY_ALL_LARGE_COLOR        148
 #define IDS_PREFS_COLOUR                148
 #define IDS_UNSAVED_CHANGES             149
-#define IDD_CUSTOMALPHABET              150
 #define IDS_PREFS_ADVANCED              150
 #define IDD_CUSTOMLETTER                151
 #define IDS_PREFS_LM                    151
-#define IDD_ADDLETTERS                  152
 #define IDS_PREFS_SOCKET                152
-#define IDD_NEWALPHABET                 153
 #define IDS_SOCKET_ERROR_TITLE          153
-#define IDD_NEWGROUP                    154
 #define IDS_ERR_SOCKET_PORT             154
-#define IDD_EDITLETTER                  155
 #define IDS_ERR_SOCKET_MINMAX           155
 #define IDD_KEYCONTROL                  156
 #define ID_Menu                         156
-#define IDD_KEYCONTROL2                 156
 #define IDS_ERR_SOCKET_TITLE            156
-#define IDD_ADDKEYS                     157
 #define ID_OPTIONS_EDITFONT             157
-#define IDD_WINSELECT                   158
 #define ID_OPTIONS_ENTERTEXT            159
-#define IDB_FINDER_EMPTY                162
 #define IDB_BITMAP1                     163
-#define IDB_FINDER_FULL                 163
 #define IDC_CURSOR1                     165
-#define IDC_SEARCH_CURSOR               165
 #define ID_OPTIONS_EDITKEYBOARDCONTROL  166
 #define IDD_PREFS                       168
 #define IDD_CONTROLPAGE                 168
 #define ID_OPTIONS_PREFS                169
-#define IDD_COLOUR                      169
 #define ID_OPTIONS_CONTROLMODE          170
 #define IDD_KEYCONTROL1                 170
 #define IDD_MISCPAGE                    170
@@ -100,92 +88,46 @@
 #define IDD_SOCKET                      174
 #define IDD_LM                          175
 #define IDD_LMPAGE                      175
-#define IDD_BUTTON_PREFS                176
-#define IDD_BUTTON_SETTINGS             177
+#define IDD_APPEARANCEPAGE              178
+#define IDD_APPPAGE                     179
 #define IDC_LIST1                       1008
 #define IDC_INPUT_LIST                  1008
 #define IDC_LIST_ALPH                   1009
 #define IDC_CONTROL_LIST                1009
 #define IDC_ALPHABETS                   1010
-#define IDC_GROUPS                      1011
 #define IDC_COMBO1                      1012
-#define IDC_CHARS                       1013
 #define IDC_BUTTON1                     1014
 #define IDC_DRAGBUTTON                  1014
+#define IDC_DFONT_BUTTON                1014
 #define IDC_ADD_ALPH                    1015
 #define IDC_BUTTON2                     1016
+#define IDC_EFONT_BUTTON                1016
 #define IDC_DEL_ALPH                    1017
 #define IDC_BUTTON3                     1018
 #define IDC_BUTTON_PREFS                1018
-#define IDC_ADD_GROUP                   1019
+#define IDC_BUTTON_PREFS2               1019
 #define IDC_CLIPTOALPH_UNORDERED        1020
 #define IDC_THICKLINE                   1020
 #define IDC_LIST2                       1021
 #define IDC_LIST_GROUP                  1022
 #define IDC_CLIPTOALPH_ORDERED          1023
-#define IDC_KCSELECT                    1023
-#define IDC_DELGROUP                    1024
 #define IDC_BUTTON4                     1025
 #define IDC_DEL_GROUP                   1026
-#define IDC_ADD_CHAR                    1027
 #define IDC_CLONE                       1028
-#define IDC_DEL_CHAR                    1029
 #define IDC_EDIT                        1030
-#define IDC_EDIT_CHAR                   1031
-#define IDC_MOVE_GROUP                  1032
-#define IDC_ENCODING                    1033
 #define IDC_MOVE_GROUP2                 1034
-#define IDC_MOVE_CHAR                   1035
 #define IDC_SPACE                       1036
-#define IDC_RO                          1037
-#define IDC_RIGHTY                      1038
-#define IDC_DISPLAY                     1039
 #define IDC_EDIT2                       1040
 #define IDC_UNIFORMVAL                  1040
 #define IDC_IDLETIME                    1040
 #define IDC_SOCKET_X_LABEL              1040
-#define IDC_TEXT                        1041
 #define IDC_SOCKET_Y_LABEL              1041
-#define IDOK_NEWALPH                    1042
-#define IDC_NEW_ALPH_STRING             1043
-#define IDOK_CUSTOMIZE                  1044
-#define IDC_DESCRIPTION                 1045
-#define IDC_TRAIN                       1046
-#define IDOK_NEWGROUP                   1047
-#define IDC_GROUP_TEXT                  1048
-#define IDOK_CHAR                       1049
-#define IDC_GROUP_BUDDY                 1050
-#define IDC_CHAR_BUDDY                  1051
-#define IDOK_ADDCHAR                    1052
-#define ID_CUSTOM_CANCEL                1054
 #define IDC_ADD_KEY                     1055
 #define IDC_DEL_KEY                     1056
 #define IDC_EDIT_KEY                    1057
 #define IDC_KEYS                        1058
-#define IDC_UPX                         1059
-#define IDC_UPY                         1060
-#define IDC_DOWNX                       1061
-#define IDC_DOWNY                       1062
-#define IDC_LEFTX                       1063
-#define IDC_LEFTY                       1064
-#define IDC_RIGHTX                      1065
-#define IDOK_KEYCONT                    1066
-#define ID_CANCEL_KEYCONT               1067
-#define IDC_DRAGCONTROL                 1068
-#define IDC_KCFORWARD                   1069
-#define IDC_KCBACK                      1070
 #define IDC_CHECK3                      1071
 #define IDC_LEFT                        1071
-#define IDC_5X                          1072
-#define IDC_5Y                          1073
-#define IDC_6X                          1074
-#define IDC_6Y                          1075
-#define IDC_7X                          1076
-#define IDC_7Y                          1077
-#define IDC_8X                          1078
-#define IDC_8Y                          1079
-#define IDC_9X                          1080
-#define IDC_9Y                          1081
 #define IDC_YPIX                        1082
 #define IDC_1D                          1084
 #define IDC_EYETRACKER                  1085
@@ -206,25 +148,16 @@
 #define IDC_SLIDER1                     1100
 #define IDC_UNIFORMSLIDER               1100
 #define IDC_ZOOMESTEPS_SLIDER           1100
-#define IDC_RIGHTZOOM_SLIDER            1100
 #define IDC_MOUSEPOSDIST                1101
-#define IDC_ZOOMESTEPS_SLIDER2          1101
-#define IDC_ZOOMSTEPS_SLIDER            1101
-#define IDC_PARAGRAPH                   1102
-#define IDC_CONTROLCHAR                 1103
 #define IDC_EDIT1                       1104
-#define IDC_COLOUR                      1104
 #define IDC_SOCKET_PORT                 1104
-#define IDC_ZOOMSTEPS_VALUE             1104
-#define IDC_RIGHTZOOM_VALUE             1105
-#define IDC_NBOXES_VALUE                1106
 #define IDC_RADIO1                      1107
-#define IDC_NU_VALUE                    1107
 #define IDC_RADIO2                      1108
 #define IDC_RADIO3                      1109
 #define IDC_RADIO4                      1110
 #define IDC_RADIO5                      1111
 #define IDC_CHECK1                      1112
+#define IDC_OR_CUSTOM                   1112
 #define IDC_CHECK2                      1113
 #define IDC_ADAPTIVE                    1114
 #define IDC_LM_PPM                      1115
@@ -253,14 +186,18 @@
 #define IDC_MENU                        1133
 #define IDC_THREE_DIRECT                1134
 #define IDC_ALTERNATING_DIRECT          1136
-#define IDC_ZOOMSTEPS_LABEL             1137
-#define IDC_NOBOXES_SLIDER              1140
-#define IDC_SAFETY_SLIDER               1141
-#define IDC_NONUNIFORM_SLIDER           1142
 #define IDC_MOUSEPOS_STYLE              1144
 #define IDC_AUTOCALIBRATE               1145
 #define IDC_CONTROLMODE                 1146
-#define IDC_SAFETY_VALUE                1146
+#define IDC_SPIN1                       1147
+#define IDC_OR_DEFAULT                  1148
+#define IDC_OR_LR                       1149
+#define IDC_OR_RL                       1150
+#define IDC_OR_TB                       1151
+#define IDC_OR_BT                       1152
+#define IDC_FONT_SMALL                  1153
+#define IDC_FONT_LARGE                  1154
+#define IDC_FONT_VLARGE                 1155
 #define ID_FILE_NEW                     32771
 #define ID_EDIT_CUT                     32772
 #define ID_EDIT_COPY                    32773
@@ -306,15 +243,16 @@
 #define ID_OPTIONS_EDITKEYS             32814
 #define ID_OPTIONS_KEYCONTROL           32815
 #define ID_OPTIONS_WINDOWPAUSE          32816
+#define ID_HELP_DASHERTUTORIAL          32817
 #define IDC_STATIC                      -1
 
 // Next default values for new objects
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
-#define _APS_NEXT_RESOURCE_VALUE        178
-#define _APS_NEXT_COMMAND_VALUE         32817
-#define _APS_NEXT_CONTROL_VALUE         1147
+#define _APS_NEXT_RESOURCE_VALUE        180
+#define _APS_NEXT_COMMAND_VALUE         32818
+#define _APS_NEXT_CONTROL_VALUE         1156
 #define _APS_NEXT_SYMED_VALUE           110
 #endif
 #endif
